import { Block } from '@minecraft/server';
// Define the namespace for custom blocks, used in naming states in blocks: 'custom:north' , 'custom:down', ...

/**
 * Class for handling directions and their opposites.
 */
class DirectionHelper {
  static OPPOSITE_DIRECTIONS = {
    'north': 'south',
    'east': 'west',
    'south': 'north',
    'west': 'east',
    'up': 'down',
    'down': 'up',
    'north_east': 'south_west',
    'east_south': 'west_north',
    'south_west': 'north_east',
    'west_north': 'east_south'
  };

  /**
   * Returns the opposite direction for the given direction.
   * 
   * @param {string} direction - The original direction.
   * @returns {string} - The opposite direction.
   */
  static getOppositeDirection(direction) {
    return this.OPPOSITE_DIRECTIONS[direction];
  };
};

/**
 * Class representing a custom block with state handling.
 */
class CustomBlock {
  /**
   * Creates an instance of CustomBlock.
   * 
   * @param {Block} block - The block to interact with.
   */
  constructor(block) {
    this.block = block;
    this.namespace = "connected_glasses";
  };

  /**
   * Retrieves the nearby blocks in different directions.
   * 
   * @returns {object} - An object containing the nearby blocks and their respective directions.
   */
  getNearbyBlocks() {
    return {
      'up': this.block.above(),
      'down': this.block.below(),
      'north': this.block.north(),
      'east': this.block.east(),
      'south': this.block.south(),
      'west': this.block.west(),
      'north_east': this.block.north().east(),
      'east_south': this.block.east().south(),
      'south_west': this.block.south().west(),
      'west_north': this.block.west().north()
    };
  };

  /**
   * Updates the state of the block with a custom state.
   * 
   * @param {string} direction - The direction in which the block is being updated.
   * @param {boolean} stateValue - The state value to apply.
   */
  updateState(direction, stateValue) {
    return this.block.setPermutation(this.block.permutation.withState(`${this.namespace}:${direction}`, stateValue));
  };
};

/**
 * Class responsible for managing connected block logic.
 */
class ConnectedBlockManager {
  /**
   * Creates an instance of ConnectedBlockManager.
   * 
   * @param {Block} block - The block that was placed or destroyed.
   * @param {string} blockId - The block's typeId or type.id.
   * @param {string} [checkType='break'] - Whether the block was placed or broken ("placed" or "break").
   */
  constructor(block, blockId, checkType = 'break') {
    this.customBlock = new CustomBlock(block);
    this.blockId = blockId;
    this.stateValue = checkType === 'placed';
  };

  /**
   * Updates the current block and the nearby blocks.
   */
  updateBlock() {
    const nearbyBlocks = this.customBlock.getNearbyBlocks();
    for (const direction in nearbyBlocks) {
      const nearbyBlock = nearbyBlocks[direction];

      if (nearbyBlock?.typeId === this.blockId) {
        // Update current block if placed
        if (this.stateValue)
          this.customBlock.updateState(direction, this.stateValue);

        // Update nearby block in the opposite direction
        const nearbyCustomBlock = new CustomBlock(nearbyBlock);
        nearbyCustomBlock.updateState(DirectionHelper.getOppositeDirection(direction), this.stateValue);
      }
    }
  };
};

/**
 * Custom glass component to handle block placement and destruction events.
 */
export class ConnectedGlassComponent {
  /**
   * Handles the block placement event.
   * 
   * @param {object} event - The event containing information about the placed block.
   */
  static onPlace({ block }) {
    const manager = new ConnectedBlockManager(block, block.typeId, 'placed');
    manager.updateBlock();
  };

  /**
   * Handles the block destruction event.
   * 
   * @param {object} event - The event containing information about the destroyed block.
   */
  static onPlayerDestroy({ block, destroyedBlockPermutation }) {
    const manager = new ConnectedBlockManager(block, destroyedBlockPermutation.type.id);
    manager.updateBlock();
  };
};


class AdvancedCustomBlock extends CustomBlock {
  /**
   * Applies a special custom state to the block.
   * 
   * @param {string} customState - The custom state to apply.
   */
  applyCustomState(customState) {
    const newBlockPermutation = this.block.permutation.withState(`${addonNamespace}:special`, customState);
    this.block.setPermutation(newBlockPermutation);
  }
}

// Consider using other events to update your block, such as ExplosionAfterEvent and PistonActivateAfterEvent